@@ -531,11 +531,22 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
         return NULL == masked || 0 == masked[idx];
 }
 
+static inline double polarisation(vec3s kf, double weight, int do_polarisation)
+{
+        if (do_polarisation){
+                CGLM_ALIGN_MAT vec3s epsilon = {{0, 1, 0}};
+                float p = glms_vec3_dot(epsilon, kf) / glms_vec3_norm2(kf);
+                weight = weight / (1 - p*p);
+        }
+        return weight;
+}
+
 #define HKL_BINOCULARS_SPACE_QCUSTOM_IMPL(image_t)			\
         HKL_BINOCULARS_SPACE_QCUSTOM_DECL(image_t)			\
         {                                                               \
 		size_t i;						\
                 HklBinocularsSpaceItem item;                            \
+                double correction;                                      \
 		const char **names = axis_name_from_subprojection(subprojection, space, n_resolutions); \
 		assert(n_pixels == space->max_items);			\
                                                                         \
@@ -587,13 +598,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v, ki);       \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
                                         item.indexes_0[0] = rint(v.raw[0] / resolutions[0]); \
                                         item.indexes_0[1] = rint(v.raw[1] / resolutions[1]); \
                                         item.indexes_0[2] = rint(v.raw[2] / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -609,13 +621,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
 					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
 					item.indexes_0[1] = rint(asin(glms_vec3_norm(v) / 2 / k) * 2 / M_PI * 180 / resolutions[1]); \
 					item.indexes_0[2] = rint(timestamp / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -631,13 +644,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
 					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
 					item.indexes_0[1] = rint(timestamp / resolutions[1]); \
 					item.indexes_0[2] = REMOVED;	\
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -653,13 +667,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
 					item.indexes_0[0] = rint(sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1]) / resolutions[0]); \
                                         item.indexes_0[1] = rint(v.raw[2] / resolutions[1]); \
 					item.indexes_0[2] = rint(timestamp / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -675,13 +690,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
 					item.indexes_0[0] = rint(sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1]) / resolutions[0]); \
 					item.indexes_0[1] = rint(v.raw[2] / resolutions[1]); \
 					item.indexes_0[2] = REMOVED;	\
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -697,13 +713,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
 					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
 					item.indexes_0[1] = rint((atan2(v.raw[2], -v.raw[1])) / M_PI * 180 / resolutions[1]); \
 					item.indexes_0[2] = rint(v.raw[0] / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -719,13 +736,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
 					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
 					item.indexes_0[1] = rint((atan2(v.raw[2], v.raw[0])) / M_PI * 180 / resolutions[1]); \
 					item.indexes_0[2] = rint(v.raw[1] / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -741,13 +759,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
 					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
 					item.indexes_0[1] = rint((atan2(v.raw[0], v.raw[1])) / M_PI * 180 / resolutions[1]); \
 					item.indexes_0[2] = rint(v.raw[2] / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -763,6 +782,7 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
@@ -770,7 +790,7 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
 					double ratio = v.raw[2] + item.indexes_0[0]; \
 					item.indexes_0[1] = rint(v.raw[0] / ratio / resolutions[1]); \
 					item.indexes_0[2] = rint(v.raw[1] / ratio / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -789,11 +809,12 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                 CGLM_ALIGN_MAT vec3s v = {{q_x[i], q_y[i], q_z[i]}}; \
                                                                         \
                                                 v = glms_mat4_mulv3(m_holder_d, v, 1); \
+                                                correction = polarisation(v, weight, do_polarisation_correction); \
                                                                         \
                                                 item.indexes_0[0] = rint(atan2(v.raw[2], sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1])) / M_PI * 180 / resolutions[0]); \
                                                 item.indexes_0[1] = rint(atan2(v.raw[1], v.raw[0]) / M_PI * 180 / resolutions[1]); \
                                                 item.indexes_0[2] = axis; \
-                                                item.intensity = rint((double)image[i] * weight); \
+                                                item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                                 if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                         darray_append(space->items, item); \
@@ -810,13 +831,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         glms_vec3_print(v, stdout);     \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         glms_mat4_print(m_holder_d, stdout); \
                                         glms_vec3_print(v, stdout);     \
                                                                         \
 					item.indexes_0[0] = rint(v.raw[0] / resolutions[0]); \
 					item.indexes_0[1] = rint(v.raw[1] / resolutions[1]); \
 					item.indexes_0[2] = rint(v.raw[2] / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -831,11 +853,12 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                         CGLM_ALIGN_MAT vec3s v = {{q_x[i], q_y[i], q_z[i]}}; \
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                                                         \
 					item.indexes_0[0] = rint(v.raw[1] / resolutions[0]); \
 					item.indexes_0[1] = rint(v.raw[2] / resolutions[1]); \
 					item.indexes_0[2] = rint(timestamp / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -851,13 +874,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
 					item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
 					item.indexes_0[1] = rint(sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1]) / resolutions[1]); \
 					item.indexes_0[2] = rint(v.raw[2] / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                 darray_append(space->items, item); \
@@ -873,13 +897,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                         v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                         v = glms_vec3_scale_as(v, k);   \
+                                        correction = polarisation(v, weight, do_polarisation_correction); \
                                         v = glms_vec3_sub(v , ki);      \
                                         v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
 					item.indexes_0[0] = rint(sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1]) / resolutions[0]); \
                                         item.indexes_0[1] = rint(v.raw[2] / resolutions[1]); \
 					item.indexes_0[2] = rint(timestamp / resolutions[2]); \
-                                        item.intensity = rint((double)image[i] * weight); \
+                                        item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                         if(v.raw[1] != 0.0){            \
                                                 if(signbit(v.raw[1]) != 0){ \
@@ -908,13 +933,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                                 v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                                 v = glms_vec3_scale_as(v, k); \
+                                                correction = polarisation(v, weight, do_polarisation_correction); \
                                                 v = glms_vec3_sub(v , ki); \
                                                 v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
                                                 item.indexes_0[0] = rint(sqrt(v.raw[0] * v.raw[0] + v.raw[1] * v.raw[1]) / resolutions[0]); \
                                                 item.indexes_0[1] = rint(v.raw[2] / resolutions[1]); \
                                                 item.indexes_0[2] = axis; \
-                                                item.intensity = rint((double)image[i] * weight); \
+                                                item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                                 if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                         darray_append(space->items, item); \
@@ -935,13 +961,14 @@ static inline int not_masked(const uint8_t *masked, size_t idx)
                                                                         \
                                                 v = glms_mat4_mulv3(m_holder_d, v, 1); \
                                                 v = glms_vec3_scale_as(v, k); \
+                                                correction = polarisation(v, weight, do_polarisation_correction); \
                                                 v = glms_vec3_sub(v , ki); \
                                                 v = glms_mat4_mulv3(m_holder_s, v, 0); \
                                                                         \
                                                 item.indexes_0[0] = rint(glms_vec3_norm(v) / resolutions[0]); \
                                                 item.indexes_0[1] = axis; \
                                                 item.indexes_0[2] = rint(asin(glms_vec3_norm(v) / 2 / k) * 2 / M_PI * 180 / resolutions[2]); \
-                                                item.intensity = rint((double)image[i] * weight); \
+                                                item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                                 if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                                         darray_append(space->items, item); \
@@ -965,7 +992,8 @@ HKL_BINOCULARS_SPACE_QCUSTOM_IMPL(uint32_t);
 #define HKL_BINOCULARS_SPACE_HKL_IMPL(image_t)                          \
         HKL_BINOCULARS_SPACE_HKL_DECL(image_t)                          \
         {                                                               \
-                size_t i;                                            \
+                size_t i;                                               \
+                double correction;                                      \
                 const char * names[] = {"H", "K", "L"};                 \
                 HklBinocularsSpaceItem item;                            \
                                                                         \
@@ -1007,13 +1035,14 @@ HKL_BINOCULARS_SPACE_QCUSTOM_IMPL(uint32_t);
                                                                         \
                                 v = glms_mat4_mulv3(m_holder_d, v, 1);  \
                                 v = glms_vec3_scale_as(v, K);           \
+                                correction = polarisation(v, weight, do_polarisation_correction); \
                                 v = glms_vec3_sub(v, ki);               \
                                 v = glms_mat4_mulv3(m_holder_s, v, 0);  \
                                                                         \
                                 item.indexes_0[0] = rint(v.raw[0] / resolutions[0]); \
                                 item.indexes_0[1] = rint(v.raw[1] / resolutions[1]); \
                                 item.indexes_0[2] = rint(v.raw[2] / resolutions[2]); \
-                                item.intensity = rint((double)image[i] * weight); \
+                                item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                 if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                         darray_append(space->items, item); \
@@ -1034,8 +1063,9 @@ HKL_BINOCULARS_SPACE_HKL_IMPL(uint32_t);
 #define HKL_BINOCULARS_SPACE_TEST_IMPL(image_t)                         \
         HKL_BINOCULARS_SPACE_TEST_DECL(image_t)                         \
         {                                                               \
-                size_t i;                                            \
+                size_t i;                                               \
                 const char * names[] = {"H", "K", "L"};                 \
+                double correction;                                      \
                 HklBinocularsSpaceItem item;                            \
                                                                         \
                 assert(ARRAY_SIZE(names) == darray_size(space->axes));  \
@@ -1076,13 +1106,14 @@ HKL_BINOCULARS_SPACE_HKL_IMPL(uint32_t);
                                                                         \
                                 v = glms_mat4_mulv3(m_holder_d, v, 1);  \
                                 v = glms_vec3_scale_as(v, K);           \
+                                correction = polarisation(v, weight, do_polarisation_correction); \
                                 v = glms_vec3_sub(v, ki);               \
                                 v = glms_mat4_mulv3(m_holder_s, v, 0);  \
                                                                         \
                                 item.indexes_0[0] = rint(v.raw[0] / resolutions[0]); \
                                 item.indexes_0[1] = rint(v.raw[1] / resolutions[1]); \
                                 item.indexes_0[2] = rint(v.raw[2] / resolutions[2]); \
-                                item.intensity = rint((double)image[i] * weight); \
+                                item.intensity = rint((double)image[i] * correction); \
                                                                         \
                                 if(TRUE == item_in_the_limits(&item, limits, n_limits)) \
                                         darray_append(space->items, item); \
